/*
 *
 *		wmmge dockapp for windowMaker
 *
 * Copyright (C) 2001 bennyben (benoit bourdin) 
 *   <bennybben@caramail.com>
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */
 
#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
	/* wmgeneral */
#include "../wmgeneral/wmgeneral.h"
	/* xlib */
#include <X11/Xlib.h>
#include <X11/xpm.h>
	/* pixmaps */
#include "wmmge-master.xpm"
#include "wmmge-mask.xbm"
#include "wmmge_master.h"
	/* wmmge */
#include "wmmge.h"

	/* global variables */
char *wmmge_sendups;
unsigned int wmmge_remaining_time;
unsigned int wmmge_clock;
bool wmmge_powerfail;
unsigned int wmmge_load;
unsigned int wmmge_conso;

	/* default values of variables */
int wmmge_refresh_delay=		500000;			/* delay of 1/2 second */
int wmmge_blink_period=			1;			/* blink every 1/2 second */
int wmmge_infos_period=			10;			/* get infos every 5 seconds */
char wmmge_psp_dir[WMMGE_PATH_MAX]=	"/usr/upsman/psp";
unsigned int wmmge_view=		0;

	/* tables */
const struct wmmge_events WMMGE_EVENTS[]={
	{Expose,	wmmge_OnExpose},
	{DestroyNotify,	wmmge_OnDestroyNotify},
	{ButtonPress,	wmmge_OnButtonPress},
	{ButtonRelease,	wmmge_OnButtonRelease},
	{-1,		NULL}
};

rckeys WMMGE_CONFIG_KEYS[]={
	{"sendups",	&wmmge_sendups},
	{NULL,		NULL}
};


int main(int argc, char **argv)
{
	openXwindow(argc, argv, wmmge_master_xpm, 
		wmmge_mask_bits, wmmge_mask_width, wmmge_mask_height);
	
	wmmge_load=100;
	get_config();
	get_infos();
	
	for(wmmge_clock=1;;wmmge_clock++) {
		if(! (wmmge_clock%wmmge_blink_period) )
			refresh_display();
		process_events();
		usleep(wmmge_refresh_delay);
		if(! (wmmge_clock%wmmge_infos_period) ) {
			get_infos();
/*
wmmge_conso--;
wmmge_remaining_time-=10;
wmmge_powerfail=1;
wmmge_load--;
*/			verif_infos();
		}
		if(wmmge_clock>WMMGE_MAX_CLOCK)
			wmmge_clock=0;
	}
	return 0;
}

void get_config()
{
		/* parse config file */
	char fname[WMMGE_FILENAME_MAX];
	
	strcpy(fname, "/etc/wmmgerc");
	parse_rcfile(fname, WMMGE_CONFIG_KEYS);
	
	sprintf(fname, "%s/wmmgerc", getenv("HOME") );
	parse_rcfile(fname, WMMGE_CONFIG_KEYS);
}

void process_events()
{
	XEvent          Event;
	int i;
	
	while (XPending(display)) {
		XNextEvent(display, &Event);
		i=0;
		while( WMMGE_EVENTS[i].exec ) {
			if( Event.type == WMMGE_EVENTS[i].event ) {
				WMMGE_EVENTS[i].exec();
				break;
			}
			i++;
		}
	}
}

void refresh_display()
{
	switch(wmmge_view) {
	case 0:
		v1_disp_back(wmmge_remaining_time<WMMGE_ALERT_DELAY && wmmge_clock%2 );
		v1_disp_remaining_time( wmmge_powerfail && wmmge_clock%2);
		v1_disp_socket( !wmmge_powerfail || !(wmmge_clock%2) );
		if(! (wmmge_remaining_time<WMMGE_ALERT_DELAY && wmmge_clock%2) )
			v1_disp_ups();
		break;
	case 1:
		v2_disp_back();
		v2_disp_remaining_time();
		v2_disp_load(wmmge_load<WMMGE_MIN_LOAD && wmmge_clock%2);
		v2_disp_conso(wmmge_conso>WMMGE_MAX_CONSO && wmmge_clock%2);
		break;
	}
	RedrawWindowXY(0, 0);
}

void get_infos()
{
		/* get infos from the ups */
		/* get status */
	wmmge_remaining_time=get_info(WMMGE_GETTIME_CMD, WMMGE_TMP_TIME_FILE);
	wmmge_powerfail=get_info(WMMGE_GETSTATUS_CMD, WMMGE_TMP_STATUS_FILE);
	wmmge_load=get_info(WMMGE_GETLOAD_CMD, WMMGE_TMP_LOAD_FILE);
	wmmge_conso=get_info(WMMGE_GETCONSO_CMD, WMMGE_TMP_CONSO_FILE);
}

int get_info(const char *cmd, const char *tmp_file)
{
		/* execute cmd on the ups and return the result */
	FILE *ftmp;
	char tcmd[WMMGE_MAX_CMD];
	unsigned int iL;
	char res[WMMGE_READ_MAX];

	snprintf(tcmd, WMMGE_MAX_CMD, "%s/%s > %s", wmmge_psp_dir, cmd, tmp_file);
	system(tcmd);
	
	iL=0;
	ftmp=fopen(tmp_file,"r");
	while(iL<WMMGE_READ_MAX-1 && !feof(ftmp) )
		fread(res+(iL++), 1, 1, ftmp);
	res[iL]='\0';
	fclose(ftmp);

	return atoi(res);
}

void verif_infos()
{
		/* verify and correct all infos */
	if(wmmge_remaining_time<0)	wmmge_remaining_time=0;
	if(wmmge_load>100)	wmmge_load=100;
	if(wmmge_load<0)	wmmge_load=0;
	if(wmmge_conso>100)	wmmge_conso=100;
	if(wmmge_conso<0)	wmmge_conso=0;
}

/******************************* callbacks *****************************/

void wmmge_OnExpose()
{
	RedrawWindowXY(0, 0);
}

void wmmge_OnDestroyNotify()
{
	XCloseDisplay(display);
	exit(0);
}

void wmmge_OnButtonPress()
{
}

void wmmge_OnButtonRelease()
{
	wmmge_view=!wmmge_view;
}


/**************************** display functions ************************/
	/**************** view 1 ****************/
void v1_disp_remaining_time(bool flash)
{
		/* displays the battery capacity (less than 99 minutes....) */
	int mn,s;
	
	mn=wmmge_remaining_time / 60;
	s=wmmge_remaining_time % 60;

	v1_disp_figure(mn/10,
		WMMGE_V1_FIGURES_DEST_X,
		WMMGE_V1_FIGURES_DEST_Y
		,flash);
	v1_disp_figure(mn%10,
		WMMGE_V1_FIGURES_DEST_X+WMMGE_V1_FIGURES_WIDTH+1,
		WMMGE_V1_FIGURES_DEST_Y
		,flash);
	
	v1_disp_figure(s/10,
		WMMGE_V1_FIGURES_DEST_X+2*WMMGE_V1_FIGURES_WIDTH+4,
		WMMGE_V1_FIGURES_DEST_Y
		,flash);
	v1_disp_figure(s%10,
		WMMGE_V1_FIGURES_DEST_X+3*WMMGE_V1_FIGURES_WIDTH+5,
		WMMGE_V1_FIGURES_DEST_Y
		,flash);
}

void v1_disp_back(bool flash)
{
		/* displays the baskground flashing or not */
	copyXPMArea( flash?WMMGE_V1_BACK_SRC_FX:WMMGE_V1_BACK_SRC_X,
		flash?WMMGE_V1_BACK_SRC_FY:WMMGE_V1_BACK_SRC_Y,
		WMMGE_V1_BACK_WIDTH,
		WMMGE_V1_BACK_HEIGHT,
		WMMGE_V1_BACK_DEST_X,
		WMMGE_V1_BACK_DEST_Y);
}

void v1_disp_figure(unsigned int n,unsigned int x, unsigned int y, bool flash)
{
		/* displays a figure (0-9) at the wanted place */

	/* copyXPMArea args: src_x, src_y, width, height, dest_x, dest_y */
	copyXPMArea( flash?WMMGE_V1_FIGURES_SRC_FX:WMMGE_V1_FIGURES_SRC_X,
		(flash?WMMGE_V1_FIGURES_SRC_FY:WMMGE_V1_FIGURES_SRC_Y)
			+WMMGE_V1_FIGURES_HEIGHT*n,
		WMMGE_V1_FIGURES_WIDTH,
		WMMGE_V1_FIGURES_HEIGHT,
		x,y);
}

void v1_disp_socket(bool flash)
{
		/* displays the socket flashing or not */
		/* if the socket is visible */
	if(flash) {
		copyXPMArea( WMMGE_V1_SOCKET_SRC_X,
			WMMGE_V1_SOCKET_SRC_Y,
			WMMGE_V1_SOCKET_WIDTH,
			WMMGE_V1_SOCKET_HEIGHT,
			WMMGE_V1_SOCKET_DEST_X,
			WMMGE_V1_SOCKET_DEST_Y);
			/* the consommation */
		copyXPMArea( WMMGE_V1_SOCKET_SRC_FX,
			WMMGE_V1_SOCKET_SRC_FY,
			(wmmge_conso*WMMGE_V1_SOCKET_WIDTH)/100,
			WMMGE_V1_SOCKET_HEIGHT,
			WMMGE_V1_SOCKET_DEST_X,
			WMMGE_V1_SOCKET_DEST_Y);
	} else
		copyXPMArea( flash?WMMGE_V1_SOCKET_SRC_X:WMMGE_V1_SOCKET_SRC_NX,
			flash?WMMGE_V1_SOCKET_SRC_Y:WMMGE_V1_SOCKET_SRC_NY,
			WMMGE_V1_SOCKET_WIDTH,
			WMMGE_V1_SOCKET_HEIGHT,
			WMMGE_V1_SOCKET_DEST_X-1,
			WMMGE_V1_SOCKET_DEST_Y);
		/* when we are not flashing, ups can be in red */
}

void v1_disp_ups()
{
		/* displays ups from the load */
	int delta;
	
	delta=(int)( (float)(wmmge_load*WMMGE_V1_UPS_HEIGHT)/100.0 );
	
	copyXPMArea( WMMGE_V1_UPS_SRC_X,
		WMMGE_V1_UPS_SRC_Y+WMMGE_V1_UPS_HEIGHT-delta,
		WMMGE_V1_UPS_WIDTH,
		delta,
		WMMGE_V1_UPS_DEST_X,
		WMMGE_V1_UPS_DEST_Y+WMMGE_V1_UPS_HEIGHT-delta);
}

	/**************** view 2 ****************/
void v2_disp_back()
{
		/* displays the baskground */
	copyXPMArea( WMMGE_V2_BACK_SRC_X,
		WMMGE_V2_BACK_SRC_Y,
		WMMGE_V2_BACK_WIDTH,
		WMMGE_V2_BACK_HEIGHT,
		WMMGE_V2_BACK_DEST_X,
		WMMGE_V2_BACK_DEST_Y);
}

void v2_disp_figure(unsigned int n,unsigned int x, unsigned int y)
{
		/* displays a figure (0-9) at the wanted place */

	/* copyXPMArea args: src_x, src_y, width, height, dest_x, dest_y */
	copyXPMArea( WMMGE_V2_FIGURES_SRC_X,
		WMMGE_V2_FIGURES_SRC_Y
			+WMMGE_V2_FIGURES_HEIGHT*n,
		WMMGE_V2_FIGURES_WIDTH,
		WMMGE_V2_FIGURES_HEIGHT,
		x,y);
}

void v2_disp_bar(unsigned int type, unsigned int n, unsigned int x, unsigned int y)
{
		/* displays a progress bar at the wanted place */
	copyXPMArea( type?WMMGE_V2_BAR_SRC_X:WMMGE_V2_RBAR_SRC_X,
		type?WMMGE_V2_BAR_SRC_Y:WMMGE_V2_RBAR_SRC_Y,
		(WMMGE_V2_BAR_WIDTH*n)/100,
		WMMGE_V2_BAR_HEIGHT,
		x,y);
}

void v2_disp_remaining_time()
{
		/* displays the battery capacity (less than 99 minutes....) */
	int mn,s;
	
	mn=wmmge_remaining_time / 60;
	s=wmmge_remaining_time % 60;
	
	mn=mn%100;	/* less than 99 minutes */
	v2_disp_figure(mn/10,
		WMMGE_V2_FIGURES_TIME_X,
		WMMGE_V2_FIGURES_TIME_Y);
	v2_disp_figure(mn%10,
		WMMGE_V2_FIGURES_TIME_X+WMMGE_V2_FIGURES_WIDTH+1,
		WMMGE_V2_FIGURES_TIME_Y);
	v2_disp_figure(s/10,
		WMMGE_V2_FIGURES_TIME_X+2*WMMGE_V2_FIGURES_WIDTH+4,
		WMMGE_V2_FIGURES_TIME_Y);
	v2_disp_figure(s%10,
		WMMGE_V2_FIGURES_TIME_X+3*WMMGE_V2_FIGURES_WIDTH+5,
		WMMGE_V2_FIGURES_TIME_Y);
}

void v2_disp_load(bool flash)
{
		/* 10 if 0 */
	v2_disp_figure( wmmge_load/100
			+10*!(wmmge_load/100),
		WMMGE_V2_FIGURES_LOAD_X,
		WMMGE_V2_FIGURES_LOAD_Y);
		/* 10 if 0 && if wmmge_load/100==0 */
	v2_disp_figure( (wmmge_load%100)/10
			+10*!((wmmge_load%100)/10)*!(wmmge_load/100),
		WMMGE_V2_FIGURES_LOAD_X+WMMGE_V2_FIGURES_WIDTH+1,
		WMMGE_V2_FIGURES_LOAD_Y);
	v2_disp_figure( wmmge_load%10,
		WMMGE_V2_FIGURES_LOAD_X+2*WMMGE_V2_FIGURES_WIDTH+2,
		WMMGE_V2_FIGURES_LOAD_Y);
	if(!flash)
		v2_disp_bar(0,wmmge_load,
			WMMGE_V2_BAR_LOAD_X,
			WMMGE_V2_BAR_LOAD_Y);
}

void v2_disp_conso(bool flash)
{
	v2_disp_figure( wmmge_conso/100
		+10*!(wmmge_conso/100),
		WMMGE_V2_FIGURES_CONSO_X,
		WMMGE_V2_FIGURES_CONSO_Y);
	v2_disp_figure( (wmmge_conso%100)/10
			+10*!((wmmge_conso%100)/10)*!(wmmge_conso/100),
		WMMGE_V2_FIGURES_CONSO_X+WMMGE_V2_FIGURES_WIDTH+1,
		WMMGE_V2_FIGURES_CONSO_Y);
	v2_disp_figure( wmmge_conso%10,
		WMMGE_V2_FIGURES_CONSO_X+2*WMMGE_V2_FIGURES_WIDTH+2,
		WMMGE_V2_FIGURES_CONSO_Y);
	if(!flash)
		v2_disp_bar(1,wmmge_conso,
			WMMGE_V2_BAR_CONSO_X,
			WMMGE_V2_BAR_CONSO_Y);
}
